xquery version "3.1";

module namespace gramm="http://grammateus.ch/gramm";

import module namespace templates="http://exist-db.org/xquery/templates";
import module namespace app="http://grammateus.ch/templates" at "app.xql";
import module namespace config="http://grammateus.ch/config" at "config.xqm";
import module namespace URI="http://grammateus.ch/URI" at "uri.xql";
import module namespace console="http://exist-db.org/xquery/console";
import module namespace kwic="http://exist-db.org/xquery/kwic";
import module namespace functx="http://www.functx.com";

declare namespace tei = "http://www.tei-c.org/ns/1.0";
declare namespace xhtml="http://www.w3.org/1999/xhtml";

(:: VARIABLES ::)
(: Mininum and Maximum papyrus dimensions :)
declare variable $gramm:width_min := min(collection("/db/apps/grammateus/data/papyri")//tei:width);
declare variable $gramm:width_max := max(collection("/db/apps/grammateus/data/papyri")//tei:width);
declare variable $gramm:height_min := min(collection("/db/apps/grammateus/data/papyri")//tei:height);
declare variable $gramm:height_max := max(collection("/db/apps/grammateus/data/papyri")//tei:height);


(:: FUNCTX ::)

(:~ returns last substring after delimiter
 : 
 : @param arg - full string
 : @param delim - delimiter string
 :)
(:declare function functx:substring-after-last:)
(:  ( $arg as xs:string? ,:)
(:    $delim as xs:string )  as xs:string {:)
(:    replace ($arg,concat('^.*',$delim),''):)
(: };:)

(:~ returns default value in case an element or attribute is empty or non-existant.
 : 
 : @param arg - item
 : @param value - a value
 :)
(:declare function functx:if-empty:)
(:  ( $arg as item()? ,:)
(:    $value as item()* )  as item()* {:)
(::)
(:  if (string($arg) != ''):)
(:  then data($arg):)
(:  else $value:)
(: } ;:)
(: :)
(:declare function functx:index-of-match-first:)
(:  ( $arg as xs:string? ,:)
(:    $pattern as xs:string )  as xs:integer? {:)
(::)
(:  if (matches($arg,$pattern)):)
(:  then string-length(tokenize($arg, $pattern)[1]) + 1:)
(:  else ():)
(: } ;:)
 
(:: GENERAL ::)

(:~ returns formatted link to a document
 : 
 : @param uri - string uri
 :)
declare function gramm:get-doc-link ($doc as node()) {
    <a href="show.html?document={functx:substring-after-last(document-uri(root($doc)), "/")}">
        {gramm:title($doc)}
    </a> 
 };

(:~ returns TM number
 : 
 : @param doc - XML EpiDoc node
 : @return <idno>
 :)
declare function gramm:tm ($doc as node()) {
    $doc//tei:idno[@type eq "TM"]
 };
 
(:~ returns Title
 : 
 : @param doc - XML EpiDoc node
 : @return string
 :)
 declare function gramm:title ($doc as node()) {
    let $series := $doc//tei:bibl/tei:title/string()
    let $volume := $doc//tei:biblScope[@unit eq "volume"]/string()
    let $number := $doc//tei:biblScope[@unit eq "numbers"]/string()
    let $side := $doc//tei:biblScope[@unit eq "side"]/string()
    let $generic := if($doc//tei:biblScope[@unit eq "generic"]) 
                    then "(" || $doc//tei:biblScope[@unit eq "generic"]/string() || ")"
                    else ()
    return
        string-join(($series, $volume, $number, $side, $generic), " ")
 };
 
(: fa-icons :)
declare function gramm:fa-info() {
    <i class="fas fa-info-circle"/>
};
declare function gramm:fa-shape-vertical() {
    <i class="far fa-file"/>
};
declare function gramm:fa-shape-horizontal() {
    <i class="far fa-file icon-rotate"/>
};
(: <i class="far fa-sticky-note"/> :)
declare function gramm:fa-shape-square() {
    <i class="far fa-square"/>
};
declare function gramm:fa-fibre-vertical() {
    <i class="fas fa-arrow-down"/>
};
declare function gramm:fa-fibre-horizontal() {
    <i class="fas fa-arrow-right"/>
};
declare function gramm:fa-search() {
    <i class="fas fa-search"/>
};
declare function gramm:fa-image() {
    <i class="fas fa-camera fa-2x"/>
};
declare function gramm:fa-question($node as node(), $model as map(*)) {
    <i class="fa fa-question-circle"></i>
};
declare function gramm:fa-times() {
    <i class="fa fa-times-circle"></i>
};
declare function gramm:fa-up() {
    <i class="fas fa-caret-up"></i>
};
declare function gramm:fa-down() {
    <i class="fas fa-caret-down"></i>
};
declare function gramm:fa-map() {
    <i class="fas fa-map"></i>
};
declare function gramm:fa-calendar() {
    <i class="far fa-calendar-alt"></i>
};
declare function gramm:fa-next() {
    <i class="fas fa-chevron-right"></i>
};
declare function gramm:fa-previous() {
    <i class="fas fa-chevron-left"></i>
};

declare function gramm:iiif-logo() {
    <img src="$app-root/resources/img/logo_iiif.png" style="height: 20px; margin: 5px;"></img>
};
declare function gramm:back-to-top($node as node(), $model as map(*)) {
    <a id="back-to-top" href="#" class="btn btn-light btn-lg back-to-top" role="button">
        <i class="fas fa-chevron-up"/>
    </a>
};
declare function gramm:back-to-top() {
    <a id="back-to-top" href="#" class="btn btn-light btn-lg back-to-top" role="button">
        <i class="fas fa-chevron-up"/>
    </a>
};

(:: EXTERNAL DATA ::)

(:~ returns <history> fragment from external HGV file
 : with place/date of origin included.
 : For doc with multiple HGV (such as TM 13), we use the first one.
 : 
 : @param doc - document node
 : @return XML <history> fragment
 :)
declare function gramm:doc-expand-history ($doc as node()){
    
    (: get ID to external history link :)
    let $hist_id := $doc//tei:history/@corresp/string() => substring-after('#')
    
    (: link to external HGV document :)
    let $ext_history := doc($config:data-root||'/external_links_history.xml')
    let $hist_link := $ext_history//id($hist_id)/tei:ref[1]/@target/string()
    
    (: fragment of external HGV document :)
    let $fragment := doc($hist_link)//tei:history
    
    return
        $fragment
 };
 
(:~ returns <textClass> fragment from external HGV file
 : with HGV subjects.
 : For doc with multiple HGV (such as TM 13), we use the first one.
 : 
 : @param doc - document node
 : @return XML <textClass> fragment
 :)
declare function gramm:doc-expand-keywords ($doc as node()){
    
    (: get ID to external history link :)
    let $hist_id := $doc//tei:history/@corresp/string() => substring-after('#')
    
    (: link to external HGV document :)
    let $ext_history := doc($config:data-root||'/external_links_history.xml')
    let $hist_link := $ext_history//id($hist_id)/tei:ref[1]/@target/string()
    
    (: fragment of external HGV document :)
    let $fragment := doc($hist_link)//tei:textClass
    
    return
        $fragment
 };
 
(:~ Get transcription file from papyri.info.
 : 
 : @param doc - document node
 : @return external XML document
 :)
declare function gramm:doc-expand-text ($doc as node()){
    
    (: get ID to external text link :)
    let $text_id := $doc//tei:div/@corresp/string() => substring-after('#')
    
    (: link to external papyri.info document :)
    let $ext_texts := doc($config:data-root||'/external_links_texts.xml')
    let $text_link := $ext_texts//id($text_id)/tei:ref/@target/string()
    
    (: papyri.info transcription file :)
    let $externaldoc := doc($text_link)

    return
        $externaldoc
};

(:~ Get link to an external image.
 : 
 : @param doc - document node
 : @return link as string
 :)
declare function gramm:get-img-link ($doc as node()){
    
    (: get ID to external image link :)
    let $img_id := $doc//tei:graphic/@corresp/string() => substring-after('#')
    
    (: we assume only one image link, or else it will return the first :)
    let $ext_facs := doc($config:data-root||'/external_links_images.xml')
    let $img_link := $ext_facs//id($img_id)/tei:ref[1]/@target/string()
    
    return
        $img_link
};

(:~ Get iiif manifest link.
 : 
 : @param doc - document node
 : @return link as string
 :)
declare function gramm:get-iiif-manifest ($doc as node()){

    (: get ID to external image link :)
    let $img_id := $doc//tei:graphic/@corresp/string() => substring-after('#')
    
    (: we assume only one image link, or else it will return the first :)
    let $ext_facs := doc($config:data-root||'/external_links_images.xml')
    let $iiif_link := $ext_facs//id($img_id)/tei:ref[@type eq 'iiif']/@target/string()
    
    return
        $iiif_link
};


(:: PLACE ::)

(:~ returns place of origin as string
 : 
 : @param history - <history> node
 : @return string
 :)
declare function gramm:origPlace($history as node()) {
    
    (: chose placeName of @subtype nome if available :)
    (: it can happen that there is more than 1 placeName @subtype nome (e.g. TM 7883) :)
    if ($history//tei:placeName[@subtype eq 'nome'][not(@cert eq 'low')]) 
    then $history//tei:placeName[@subtype eq 'nome'][not(@cert eq 'low')][1]/string()
    
    (: otherwise take the nome in parenthesis within origPlace :)
    else if (matches($history//tei:origPlace/string(), '\([a-zA-Z\s]+\??\)'))
    then 
        (: find substring starting at the point of the match :)
        substring($history//tei:origPlace/string(), functx:index-of-match-first($history//tei:origPlace/string(), '\([a-zA-Z\s]+\??\)')) 
        (: remove parentheses :)
        => substring-before(')') 
        => substring-after('(')
    
    (: if no nome can be reliably found, then return the content of origPlace :)
    else $history//tei:origPlace/string()
    => functx:if-empty('unknown')
    => replace('unbekannt', 'unknown')
 };
 
(:: SHAPE ::)

(:~ returns shape based on ratio
 : 
 : @param ratio - double number
 :)
declare function gramm:shape ($ratio as xs:double) {
    let $shape := 
        if ($ratio < 0.8) then "Vertical"
        else if ($ratio > 1.2) then "Horizontal"
        else "Squarish"
    return
        $shape 
 };

(:~ returns shape symbol based on ratio
 : 
 : @param shape - string from gramm:fibres direction
 :)
declare function gramm:shape-symbol($shape) {
    let $symbol :=
        if ($shape eq 'Vertical') then gramm:fa-shape-vertical()
        else if ($shape eq 'Horizontal') then gramm:fa-shape-horizontal()
        else gramm:fa-shape-square()
    return
        $symbol
};

(:: FIBRES ::)

(:~ returns fibres direction
 : 
 : @param f - string from @ana of <support>
 :)
declare function gramm:fibres($doc) {
    let $f := $doc//tei:support/@ana
    let $direction :=
        if (functx:substring-after-last($f, '#') eq 'fv') then 'Vertical' 
        else if (functx:substring-after-last($f, '#') eq 'fh') then 'Horizontal'
        else 'Mixed'
    return
        $direction
};
(:~ returns fibres symbol based on direction
 : 
 : @param fibres - string from gramm:fibres direction
 :)
declare function gramm:fibres-symbol($fibres) {
    let $symbol :=
        if ($fibres eq 'Vertical') then gramm:fa-fibre-vertical()
        else if ($fibres eq 'Horizontal') then gramm:fa-fibre-horizontal()
        else <span>{gramm:fa-fibre-vertical()} &amp; {gramm:fa-fibre-horizontal()}</span>
    return
        $symbol
};

(:: DATE functions ::)

(:~ filters a list of papyri for the ones dated within a date range.
 : The date range of a papyrus must overlap with the filter date range.
 : 
 : @param min - string for a year, terminus post quem
 : @param max - string for a year, terminus ante quem
 : @param papyri - set of nodes for papyri
 : @return xs:date
 :)
declare function gramm:date-filter($min, $max, $papyri) {
    let $tpq := xs:date(gramm:get-valid-year($min, "tpq") || "-01-01")
    let $taq := xs:date(gramm:get-valid-year($max, "taq") || "-12-31")
    for $p in $papyri
    let $date := $p//tei:origDate[1]
    return 
        if (gramm:date-in-range($tpq, $taq, $date)) then $p else ()
    
};

(:~ returns a valid year, i.e. a year with four digits,
 : from terminus post/ante quem in the date filter on the papyri page.
 : 
 : @param num - string, 1 or 3-digit year
 : @return 4-digit year string
 :)
declare function gramm:get-valid-year($num, $limit) {
    (: 1-digit year 0 :)
    if ($num eq "0" and $limit eq "tpq") then
        "0001"
    else if ($num eq "0" and $limit eq "taq") then
        (: should be 0000, but not working :)
        "-0001"
    (: 3-digit year :)
    else if (matches($num, "-")) 
    then $num => replace("-", "-0") 
    else "0" || $num
};

(:~ checks if the origDate is within a date range.
 : 
 : @param min - xs:date, terminus post quem
 : @param max - xs:date, terminus ante quem
 : @param origDate - <origDate> node
 : @return returns boolean
 :)
declare function gramm:date-in-range($min, $max, $origDate) {
    if ($origDate/@when) then 
        $min <= gramm:string-to-date($origDate/@when/string()) and gramm:string-to-date($origDate/@when/string()) <= $max
    else if ($origDate[@notBefore] and $origDate[@notAfter]) then 
        let $taq := gramm:string-to-date($origDate/@notBefore/string())
        let $tpq := gramm:string-to-date($origDate/@notAfter/string())
        return
        ($min <= $taq and $tpq <= $max) or
        ($taq < $max and $tpq > $max) or
        ($min < $tpq and $taq < $min)
    else if ($origDate[@notBefore]) then 
        let $taq := gramm:string-to-date($origDate/@notBefore/string())
        let $tpq := $taq + xs:yearMonthDuration("P50Y")
        return
        ($min <= $taq and $tpq <= $max) or
        ($taq < $max and $tpq > $max) or
        ($min < $tpq and $taq < $min)
    else if ($origDate[@notAfter]) then
        let $tpq := gramm:string-to-date($origDate/@notAfter/string())
        let $taq := $tpq - xs:yearMonthDuration("P50Y")
        return
        ($min <= $taq and $tpq <= $max) or
        ($taq < $max and $tpq > $max) or
        ($min < $tpq and $taq < $min)
    else ()
};

(:~ returns date. 
 : For string containing only a year, or a year and month, the date is completed with the first day (of January).
 : 
 : @param string - string from @when, @notAfter or @notBefore of <origDate>
 : @return xs:date
 :)
declare function gramm:string-to-date($string) {
    (: yyyy-mm-dd can be converted to a date :)
    if (matches($string, "\d{4}-\d{2}-\d{2}")) 
    then xs:date($string)
    (: yyyy-mm :)
    else if (matches($string, "\d{4}-\d{2}")) 
    then xs:date(concat($string, "-01"))
    (: yyyy :)
    else xs:date(concat($string, "-01-01"))
};


(:: GENERAL SEARCH RESULTS ::)

(:~ returns search result from top-right search form
 :
 : @param hit - node from a hit (either XML or XHTML)
 : @param link - document-uri for the hit node
 :)
declare function gramm:result($hit, $document as xs:string) {
    (: get proper link to files depending on where the file is. Not ideal. :)
    let $link :=
        (: html pages :)
        if ($document => ends-with(".html")) 
        then replace($document, "/db/", "/exist/")
        (: XML files - intro, descr, class 
        (bibliography cannot be searched properly for now as it is generated dynamically :)
        else if ($document => ends-with("intro_overview.xml")) then "overview.html"
        else if ($document => ends-with("intro_concepts.xml")) then "concepts.html"
        else if ($document => matches($URI:descr)) then "description.html?d=" || functx:substring-after-last($document, "/")
        else "classification.html?d=" || functx:substring-after-last($document, "/")
    (: page title (needs refactoring?) :)
    let $title :=
        if ($document => ends-with("papyri.html")) then "Papyri"
        else if ($document => ends-with("index.html")) then "Homepage"
        else if ($document => ends-with(".html")) then doc($document)//xhtml:h1/string()
        else doc($document)//tei:body/tei:head/string()
    return
        <div class="result border-bottom">
            <a href="{$link}" target="_blank">{$title}</a>
            {(: get the search term in context (300 characters before/after) :)
            let $config := <config xmlns="" width="300"/>
            for $match in $hit
                for $m in $match//exist:match
                return
                    kwic:get-summary($match, $m, $config)
            }

        </div>
};


(:: TYPOLOGY ::)

(:~ Get document or format category as a string.
 : 
 : @param catRef - <catRef> node
 : @return string
 :)
declare function gramm:get-catDesc($catRef as node()) {
     let $scheme := $catRef/@scheme => substring-after("#")
     let $cat := $catRef/@target => substring-after("#")
     let $taxonomy := doc("/db/apps/grammateus/data/grammateus_taxonomy.xml")
     return
(:        collection($config:data-root)//id($scheme)//id($cat)/tei:catDesc:)
        $taxonomy//id($cat)/tei:catDesc
 };
 
(:~ Get link to category description.
 : 
 : @param catRef - <catRef> node
 : @return string
 :)
declare function gramm:get-descr-link($catRef as node()) {
     let $scheme := $catRef/@scheme => substring-after("#")
     let $cat := $catRef/@target => substring-after("#")
     return
        collection($config:data-root)//id($scheme)//id($cat)/tei:catDesc/tei:ptr[@type eq "descr"]/@target/string()
 };
 
 (:~ Get link to category classification.
 : 
 : @param catRef - <catRef> node
 : @return string
 :)
declare function gramm:get-class-link($catRef as node()) {
     let $scheme := $catRef/@scheme => substring-after("#")
     let $cat := $catRef/@target => substring-after("#")
     return
        collection($config:data-root)//id($scheme)//id($cat)/tei:catDesc/tei:ptr[@type eq "class"]/@target/string()
 };

(:~ returns sidebar navigation for typology/taxonomy pages
 : 
 : @param model - map containing name (string), 
 :              document(node), 
 :              nav (node) 
 :              and nav-title (string)
 : @return html <nav> fragment
 :)
declare function gramm:sidebar($model as map(*)) {
    (: document to display :)
    let $doc := $model("document")
    
    (: page title and links - either description or classification :)
    let $nav_title := $model("nav-title")
    let $page_link := $model("nav-link")
    
    (: taxonomy file :)
    let $taxonomy := $model("taxonomy")
    
    return
    <nav class="sidebar {lower-case($nav_title)}" id="sidebarSupportedContent">
        <div class="sidebar-item title">
            <a class="sidebar-link" href="{$page_link}">
                {$nav_title}
            </a>
        </div>
        {   
            (: because of categories recursion, it is easier to create the sidebar through xslt :)
            let $xsl := doc("/db/apps/grammateus/resources/xslt/sidebar.xslt")
            let $params :=
                <parameters>
                    <param name="nav" value="{$page_link}"/>
                </parameters>
            let $sidebar := transform:transform($taxonomy, $xsl, $params)
            return
                $sidebar
        }
    </nav>
};