xquery version "3.1";

import module namespace config="http://grammateus.ch/config" at "modules/config.xqm";
import module namespace URI="http://grammateus.ch/URI" at "modules/URI.xql";
import module namespace gramm="http://grammateus.ch/gramm" at "modules/gramm.xql";
import module namespace show="http://grammateus.ch/show" at "modules/show.xql";

declare namespace tei = "http://www.tei-c.org/ns/1.0";
declare namespace functx = 'http://www.functx.com';
declare namespace json="http://www.json.org";

declare option exist:serialize "method=html media-type=text/html";


declare function functx:substring-after-last
  ( $arg as xs:string? ,
    $delim as xs:string )  as xs:string {
    replace ($arg,concat('^.*',$delim),'')
 };
 
declare function functx:add-attributes
  ( $elements as element()* ,
    $attrNames as xs:QName* ,
    $attrValues as xs:anyAtomicType* )  as element()? {

   for $element in $elements
   return element { node-name($element)}
                  { for $attrName at $seq in $attrNames
                    return if ($element/@*[node-name(.) = $attrName])
                           then ()
                           else attribute {$attrName}
                                          {$attrValues[$seq]},
                    $element/@*,
                    $element/node() }
} ;


(:let $doc := collection("/db/apps/grammateus/data/bibl"):)
(::)
(::)
(::)
(:return count($doc):)

(: FIND DATES WITH ONLY nB OR nA :)
(:for $doc in collection("/db/apps/grammateus/data/papyri/"):)
(:where $doc//tei:origDate[1][(not(@notAfter) and @notBefore)or (not(@notBefore) and @notAfter)]:)
(:return:)
(:    $doc//tei:origDate[1]:)


(:: TEST CATEGORY ::)
(: $doc//tei:teiHeader//tei:catRef[@n eq 'format-type'] :)
let $doc := doc('/db/apps/grammateus/data/papyri/11965.xml')
let $catref := $doc//tei:teiHeader//tei:catRef[@n eq 'format-type']
let $target := $catref/@target => substring-after("#")
(:let $cat := gramm:category($catref):)
let $taxonomy := doc($URI:taxonomy)
let $category := $taxonomy//tei:category[@xml:id eq $target]/tei:catDesc[@n eq "short"]
for $doc in collection('/db/apps/grammateus/data/papyri')//tei:TEI
return 
   gramm:get-catDesc($doc//tei:teiHeader//tei:catRef[@n eq 'format-type'])/normalize-space()

(: FIND INTERNAL TM NB in a description file :)
(:let $doc := doc("/db/apps/grammateus/data/descriptions/lists_registers.xml"):)
(:for $ref in distinct-values($doc//tei:text//tei:ref[@type eq "external"]/string()):)
(:return :)
(:    if (doc-available(concat("/db/apps/grammateus/data/papyri/", $ref, ".xml"))) then <li>{$ref}</li>:)
(:    else ():)
    
(: return :)
(:    if (doc-available(concat("/db/apps/grammateus/data/papyri/", $ref, ".xml"))) :)
(:    then update replace $doc//tei:ref[string() eq $ref] with <ref type="internal" target="{$ref}.xml">{$ref}</ref>:)
(:    else ():)

(: CONCAT DESCRIPTIONS :)
(:let $document := xs:string("lists_registers.xml"):)
(:    :)
(:    (: get xml file as doc :):)
(:    let $doc := doc(replace(concat($config:app-root,"/data/descriptions/",$document), '/exist/', '/db/')):)
(:    :)
(:    (: transform to html :):)
(:    let $xsl := doc(concat($config:app-root, "/resources/xslt/typologyToHTML.xslt")):)
(:    let $html_typology := transform:transform($doc, $xsl, ()):)
(:    :)
(:    (: get all files for sidebar menu :):)
(:    let $descr := collection($URI:descr):)
(:    :)
(:    (:create sidebar navigation:):)
(:    let $sidebar := gramm:sidebar($descr//tei:body/tei:head):)
(:    :)
(:    return:)
(:        $doc:)
(:        <div class="row">:)
(:            <div class="col-md-3 sidebar-container">:)
(:                {$sidebar}:)
(:            </div>:)
(:            <main class="col-md-9">:)
(:                {$html_typology}:)
(:            </main>:)
(:        </div>:)
    

(: OUTPUT HGV KEYWORDS :)

(:let $synchoresis := ("16537",:)
(:"18494",:)
(:"18498",:)
(:"18499",:)
(:"18500",:)
(:"18502",:)
(:"18503",:)
(:"18504",:)
(:"18539",:)
(:"18540",:)
(:"18541",:)
(:"18548",:)
(:"18556",:)
(:"18557",:)
(:"18561",:)
(:"18562",:)
(:"18566",:)
(:"18569",:)
(:"18573",:)
(:"18593",:)
(:"18594",:)
(:"18597",:)
(:"18601",:)
(:"18606",:)
(:"18614",:)
(:"18615",:)
(:"18621",:)
(:"18628",:)
(:"20007",:)
(:"20054",:)
(:"20057",:)
(:"20539",:)
(:"20990",:)
(:"78618"):)
(::)
(:for $tm in $synchoresis:)
(:let $hgv := doc("http://papyri.info/hgv/"||$tm||"/source"):)
(:order by $tm:)
(:return:)
(:    <li>{string-join($hgv//tei:term, ",")}</li>:)
    


(:let $list :=:)
(:    for $doc in collection('/db/apps/grammateus/data/papyri')//tei:TEI[contains(descendant::tei:keywords[@n eq 'text-subtype']/tei:term/string(), 'List')]:)
(:    let $hgv := gramm:doc-expand-keywords($doc):)
(:    order by number(root($doc)//tei:idno[@type eq 'TM']/string()):)
(:    return string-join($hgv//tei:term, ','):)
(::)
(:return:)
(:    <html>:)
(:    <body>:)
(:    <table>:)
(:        {for $item in $list:)
(:        return:)
(:        <tr><td>{$item}</td></tr>}:)
(:    </table>:)
(:    </body>:)
(:    </html>:)

(: OUTPUT BIBL HGV :)
(:declare function local:get-hgv ($doc as node()){:)
(:    (: find link to external doc :):)
(:    let $ext_history := doc(concat($config:data-root, '/external_links_history.xml')):)
(:    let $history := $doc//tei:history:)
(:    let $hist_id := substring-after($history/@corresp/string(), '#'):)
(:    let $hist_link := $ext_history//tei:item[@xml:id eq $hist_id]/tei:ref[1]/@target/string():)
(:    let $externaldoc := doc($hist_link):)
(:    let $fragment := $externaldoc//tei:history:)
(:    return:)
(:        $externaldoc:)
(: };    :)
(:let $list :=:)
(:    for $doc in collection('/db/apps/grammateus/data/papyri')//tei:TEI[contains(descendant::tei:keywords[@n eq 'text-subtype']/tei:term/string(), 'Hypomnema')]:)
(:    let $hgv := local:get-hgv($doc):)
(:    order by number(root($doc)//tei:idno[@type eq 'TM']/string()):)
(:    return :)
(:        <item>:)
(:        <series>{$hgv//tei:bibl[@subtype eq "principal"]/tei:title[@type eq "abbreviated"]/text()}</series>:)
(:        <volume>{$hgv//tei:bibl[@subtype eq "principal"]/tei:biblScope[@type eq "volume"]/text()}</volume>:)
(:        <fascicle>{$hgv//tei:bibl[@subtype eq "principal"]/tei:biblScope[@type eq "fascicle"]/text()}</fascicle>:)
(:        <number>{$hgv//tei:bibl[@subtype eq "principal"]/tei:biblScope[@type eq "numbers"]/text()}</number>:)
(:        <generic>{$hgv//tei:bibl[@subtype eq "principal"]/tei:biblScope[@type eq "generic"]/text()}</generic>:)
(:        <page>{$hgv//tei:bibl[@subtype eq "principal"]/tei:biblScope[@type eq "page"]/text()}</page>:)
(:        </item>:)
(:return:)
(:    <html>:)
(:    <body>:)
(:    <table>:)
(:        <tr>:)
(:            <th>Series</th>:)
(:            <th>Vol</th>:)
(:            <th>Fasc</th>:)
(:            <th>Num</th>:)
(:            <th>Generic</th>:)
(:            <th>Page</th>:)
(:        </tr>:)
(:        {for $item in $list:)
(:        return:)
(:        <tr>{for $i in $item:)
(:            return:)
(:                <td>{$i}</td>:)
(:        }</tr>}:)
(:    </table>:)
(:    </body>:)
(:    </html>:)

    
(: COUNT LINES/COL/MAX LINES etc :)

(:let $doc := doc('/db/apps/grammateus/data/papyri/11965.xml'):)
(::)
(:    let $w := $doc//tei:width/text():)
(:    let $h := $doc//tei:height/text():)
(:    :)
(:(: text content :):)
(:    let $text_xml := gramm:doc-expand-text($doc) (: papyri.info xml :):)
(:    let $text_html := show:epidoc($text_xml) (: epidoc html :):)
(:    let $sections := $doc//tei:ab[@type eq 'section']:)
(:    let $text_sections := show:text-content($sections, $text_html) (: text sections bg color :):)
(:    let $txt := $text_sections/div:)
(::)
(:    (: text container :):)
(:    let $columns := $doc//tei:layout/@columns/string():)
(:    let $nb_col := count($text_sections//div[@id eq 'edition']/div[@class eq "textpart"][not(preceding-sibling::span[contains(@id, 'abv')])]):)
(:    (: +1 => 1st line is <a> instead of <br>:)
(:        we do not count lines on the verso :):)
(:    let $lines := count($text_sections//div[@id eq 'edition']//br[not(parent::div[preceding-sibling::span[contains(@id, 'v')]])])+1:)
(:    let $max_lines :=:)
(:        for $textpart in $text_sections//div[@id eq 'edition']//div[@class eq 'textpart']:)
(:        let $l := count($textpart//br[not(parent::div[preceding-sibling::span[contains(@id, 'v')]])])+1:)
(:        order by $l descending:)
(:        return $l:)
(:(:    let $style := show:txt-container-style($w, $ratio, $columns, $lines, $max_lines[1]):):)
(:(: there is 12 $columns, but 64 $textpart; 310 $lines, but 64x1 $max_lines, there should be a total of 310 lines divided into 64 textparts  :):)
(: :)
(: (: svg variables :):)
(:     let $factor := number("20"):)
(:    let $factor_font :=  number("38"):)
(:    let $factor_line_height := number("15"):)
(:    let $factor_space_dim := number("30"):)
(:    let $max_str_length := :)
(:(:        for $column in $txt//div[@class eq 'textpart']:):)
(:(:        return:):)
(:(:            max($column//span[contains(@class, "section")]/string-length(.)):):)
(:max($txt//div[@class eq 'textpart']//span[contains(@class, "section")]/string-length(.))*number($nb_col):)
(:    let $ratio_width := $w div sum($max_str_length):)
(:    let $ratio_height := $h div max($max_lines):)
(:    let $font_size := $ratio_width*$factor_font:)
(:    let $line_height := ($ratio_height div $font_size)*$factor_line_height:)
(:    :)
(:    let $debug_msg :=:)
(:        <msg>:)
(:            <item>{$max_str_length}</item>:)
(:            <item>{$ratio_width}</item>:)
(:            <item>{$ratio_height}</item>:)
(:            <item>{$font-size}</item>:)
(:            <item>{$line_height}</item>:)
(:        </msg>:)
(::)
(:return:)
(:    $max_lines:)


(: FORMAT DATE :)
(:let $date := "-0260-11-23":)
(:let $year := year-from-date(xs:date($date)):)
(:let $century :=show:get-century-from-date($date):)
(:return $century:)

(:declare function local:get-century-from-date($date) {:)
(:    let $year := year-from-date(xs:date($date)):)
(:    let $century :=:)
(:        if(0001 <= $year and $year <= 0100) then:)
(:            'I':)
(:        else if(0101 <= $year and $year <= 0200) then:)
(:            'II':)
(:        else if(0201 <= $year and $year <= 0300) then:)
(:            'III':)
(:        else if(0301 <= $year and $year <= 0400) then:)
(:            'IV':)
(:        else if(0401 <= $year and $year <= 0500) then:)
(:            'V':)
(:        else if(0501 <= $year and $year <= 0600) then:)
(:            'VI':)
(:        else if(0601 <= $year and $year <= 0700) then:)
(:            'VII':)
(:        else if(0701 <= $year and $year <= 0800) then:)
(:            'VIII':)
(:        else():)
(:        :)
(:    return $century:)
(:};:)
(::)
(:declare function local:get-era-from-date($date) {:)
(:    let $era := :)
(:        if ($date < xs:date("0001-01-01")) then 'BCE' else 'CE':)
(:            :)
(:    return $era:)
(:};:)
(::)
(:declare function local:string-to-date($origDate) {:)
(::)
(:    let $when :=:)
(:        if (matches($origDate/@when/string(), "\d{4}-\d{2}-\d{2}")) then xs:date($origDate/@when/string()):)
(:        else if (matches($origDate/@when/string(), "\d{4}-\d{2}")) then xs:date(concat($origDate/@when/string(), "-01")):)
(:        else if ($origDate/@when) then xs:date(concat($origDate/@when/string(), "-01-01")):)
(:        else ():)
(:    :)
(:    let $notAfter :=:)
(:        if (matches($origDate/@notAfter/string(), "\d{4}-\d{2}-\d{2}")) then xs:date($origDate/@notAfter/string()):)
(:        else if (matches($origDate/@notAfter/string(), "\d{4}-\d{2}")) then xs:date(concat($origDate/@notAfter/string(), "-01")):)
(:        else if ($origDate/@notAfter) then xs:date(concat($origDate/@notAfter/string(), "-01-01")):)
(:        else ():)
(:        :)
(:    let $notBefore :=:)
(:        if (matches($origDate/@notBefore/string(), "\d{4}-\d{2}-\d{2}")) then xs:date($origDate/@notBefore/string()):)
(:        else if (matches($origDate/@notBefore/string(), "\d{4}-\d{2}")) then xs:date(concat($origDate/@notBefore/string(), "-01")):)
(:        else if ($origDate/@notBefore) then xs:date(concat($origDate/@notBefore/string(), "-01-01")):)
(:        else ():)
(:   :)
(:    return :)
(:        ($when, $notBefore, $notAfter):)
(:};:)
(::)
(:declare function local:format-date($date) {:)
(:    let $century := show:get-century-from-date($date):)
(:    let $era := show:get-era-from-date($date):)
(:    return:)
(:        concat($century, ' ', $era):)
(:};:)
(:declare function local:date($doc as node()){:)
(:    let $hgv_fragment := gramm:doc-expand-history($doc):)
(::)
(:    (: select the first of possible origDate elements :):)
(:    let $origDate := $hgv_fragment//tei:origDate[1]:)
(:    :)
(:    let $date := local:string-to-date($origDate):)
(::)
(:    (: very ugly way to deal with date :):)
(::)
(:    let $formatted_date :=:)
(:        (: deal with @when attribute :):)
(:        if ($origDate[@when]) then:)
(:            local:format-date($date[1]):)
(:        (: deal with @notBefore and @notAfter :):)
(:        else if ($origDate[@notBefore] and $origDate[@notAfter]) then:)
(:            (: @notBefore :):)
(:            let $nb_date := local:format-date($date[2]):)
(:            (: @notAfter :):)
(:            let $na_date := local:format-date($date[3]):)
(:            return if ($nb_date != $na_date):)
(:                then concat($nb_date, " - ", $na_date):)
(:                else $nb_date:)
(:        (: deal with only @notBefore or @notAfter :):)
(:        else if ($origDate[@notBefore]) then:)
(:            local:format-date($date[2]):)
(:        else if ($origDate[@notAfter]) then:)
(:            local:format-date($date[3]):)
(:        else ():)
(:    return $formatted_date:)
(:};:)
(::)
(:(:  test with TM 28932 :):)
(:let $doc := doc('/db/apps/grammateus/data/papyri/28932.xml'):)
(:let $hgv_fragment := gramm:doc-expand-history($doc):)
(:let $origDate := $hgv_fragment//tei:origDate[1]:)
(:let $date := local:string-to-date($origDate):)
(::)
(:return:)
(:    local:format-date($date[2]):)

(:: COUNT TYPES/SUBTYPES ::)
(:for $type in distinct-values(collection("/db/apps/grammateus/data/papyri")//tei:keywords[@n eq 'text-type']/tei:term):)
(:return:)
(:    <li>{$type}:)
(:        <ul>{:)
(:            for $doc in collection("/db/apps/grammateus/data/papyri"):)
(:        where $doc//tei:keywords[@n eq 'text-type'][tei:term/text() eq $type]:)
(:        let $subtype := $doc//tei:keywords[@n eq 'text-subtype']/tei:term:)
(:        order by $subtype:)
(:        return:)
(:            <li>{$subtype/text()}</li>:)
(:        }</ul>:)
(:    </li>:)

(:for $subtype in distinct-values(collection("/db/apps/grammateus/data/papyri")//tei:keywords[@n eq 'text-subtype']/tei:term):)
(:return:)
(:    <li>{$subtype}</li>:)
    
(: COUNT IIIF :)
(:for $iiif in doc("/db/apps/grammateus/data/external_links_images.xml")//tei:ref[@type eq 'iiif']:)
(:return:)
(:    $iiif:)

(:: MULTI IMG LINKS ::)
(:let $doc := doc('/db/apps/grammateus/data/papyri/11214.xml'):)
(:let $img_link := gramm:get-img-link($doc):)
(:return $img_link:)

(:  $lines > 26 and $ratio < 0.35 :)
(:for $doc in collection($URI:papyri):)
(:let $text_xml := gramm:doc-expand-text($doc):)
(:let $text_html := show:epidoc($text_xml):)
(:let $sections := $doc//tei:ab[@type eq 'section']:)
(:let $text_sections := show:text-content($sections, $text_html):)
(:let $lines := count($text_sections//div[@id eq 'edition']//br[not(ancestor::div[preceding-sibling::span[contains(@id, 'v')]])])+1:)
(:let $ratio := $doc//tei:width div $doc//tei:height:)
(:let $tm := $doc//tei:idno[@type eq 'TM']:)
(:order by $tm:)
(:return:)
(:    (if ($lines > 26 and $lines <= 33 and $ratio <0.42) then <li>TM {$tm} / {gramm:get-doc-link($doc)}</li> else ()):)
(: <li>{gramm:get-doc-link($doc)}</li>:)

(: show vertical ratio/line nb 3598 :)
(:let $doc := doc('/db/apps/grammateus/data/papyri/11738.xml'):)
(:(: shape :):)
(:    let $w := $doc//tei:width/text():)
(:    let $h := $doc//tei:height/text():)
(:    let $ratio := $w div $h:)
(:    let $shape := gramm:shape($ratio):)
(:    (: text content :):)
(:    let $text_xml := gramm:doc-expand-text($doc):)
(:    let $text_html := show:epidoc($text_xml):)
(:    (: add text sections background colour :):)
(:    let $sections := $doc//tei:ab[@type eq 'section']:)
(:    (: sections xslt transformation :):)
(:    let $text_sections := show:text-content($sections, $text_html):)
(:    let $lines := count($text_sections//div[@id eq 'edition']//br[not(ancestor::div[preceding-sibling::span[contains(@id, 'v')]])])+1 (: 1st line is <a> instead of <br> :):)
(:    let $max_lines := :)
(:        for $textpart in $text_sections//div[@id eq 'edition']//div[@class eq 'textpart']:)
(:        let $l := count($textpart//br[not(ancestor::div[preceding-sibling::span[contains(@id, 'v')]])]):)
(:        order by $l descending:)
(:        return $l:)
(:return ($max_lines[1], $lines, $ratio, $text_sections):)

(: MULTI COLUMNS :)
(:for $doc in collection($URI:papyri):)
(:where $doc//tei:layout[@columns > 1]:)
(:return:)
(:    $doc//tei:idno[@type eq 'TM']:)


(: VERTICAL fibres, NO VERSO :)
(:for $doc in collection($URI:papyri):)
(:where gramm:fibres($doc) eq 'Vertical' and $doc//tei:term/text() eq 'Petition':)
(:let $edition := gramm:doc-expand-text($doc):)
(:return:)
(:    if ($edition//tei:div[contains(@n, 'v')]) then:)
(:        ():)
(:    else:)
(:        <li>{$doc//tei:idno[@type eq 'TM']/text()}</li>:)

(: SVG IMG :)
(:{$w} {$h}:)
(:declare function local:svg($w, $h, $txt) {:)
(:    <svg viewbox="0 0 50 50" preserveAspectRatio="xMinYMin meet":)
(:        xmlns="http://www.w3.org/2000/svg">:)
(:        <!--<g x="0" y="0">-->:)
(:            <rect width="{$w}" height="{$h}" style="fill:rgba(234, 222, 204, 0.5);">:)
(:                :)
(:            </rect>:)
(:            <!-- plain text is working, not html-->:)
(:            <text x="0" y="1" font-family="Verdana" font-size="1" fill="blue">TESTING</text>:)
(:            <foreignObject width="{$w}" height="{$h}" style="font:1px serif;color:red;">:)
(:                <!--<div xmlns="http://www.w3.org/1999/xhtml">width="100%" height="100%"Lorem ipsum dolor sit amet, consectetur adipiscing elit. Lorem ipsum dolor sit amet, consectetur adipiscing elit. Lorem ipsum dolor sit amet, consectetur adipiscing elit. Lorem ipsum dolor sit amet, consectetur adipiscing elit. Lorem ipsum dolor sit amet, consectetur adipiscing elit. Lorem ipsum dolor sit amet, consectetur adipiscing elit.</div>-->:)
(:                {$txt}:)
(:            </foreignObject>:)
(:        <!--</g>-->:)
(:    </svg>:)
(:};:)
(::)
(:let $doc := doc('/db/apps/grammateus/data/papyri/8802.xml'):)
(::)
(:(: get xml file as doc :):)
(:    :)
(:    (: general variables :):)
(:    let $tm := $doc//tei:idno[@type eq 'TM']:)
(:    let $type := $doc//tei:keywords[@n eq 'text-type']:)
(:    let $subtype := $doc//tei:keywords[@n eq 'text-subtype']:)
(:    :)
(:    (: fibres :):)
(:    let $fibres := functx:substring-after-last($doc//tei:support/@ana/string(), '#'):)
(:    :)
(:    (: shape :):)
(:    let $w := $doc//tei:width/text():)
(:    let $h := $doc//tei:height/text():)
(:    let $ratio := $w div $h:)
(:    let $shape := gramm:shape($ratio):)
(:    :)
(:    (: link to image :):)
(:    let $img_link := gramm:get-img-link($doc):)
(:    :)
(:    (:: text-container and text content ::):)
(:    :)
(:    (: text content :):)
(:    let $text_xml := gramm:doc-expand-text($doc):)
(:    let $text_html := show:epidoc($text_xml):)
(:    (: add text sections background colour :):)
(:    let $sections := $doc//tei:ab[@type eq 'section']:)
(:    (: sections xslt transformation :):)
(:(:    let $xml := <root>{$sections, $text_html}</root>:):)
(:(:    let $xsl := doc(concat($config:app-root, "/resources/xslt/txt-sect.xslt")):):)
(:(:    let $text_sections := transform:transform($xml, $xsl, ()):):)
(:    let $text_sections := show:text-content($sections, $text_html):)
(:    :)
(:    (: text container :):)
(:    (: TODO: add max-width and max-height properties to style? :):)
(:    let $columns := $doc//tei:layout/@columns/string():)
(:    let $lines := count($text_sections//div[@id eq 'edition']//br)+1 :)
(:    (: maybe I should change the way to count. this counts lb but not = br in html (and not same as how lines appear in final viz):):)
(:    let $style := show:txt-container-style($ratio, $columns, $lines):)
(:   :)
(::)
(:    :)
(:    return:)
(:    <div>:)
(:        <div class="page-header">:)
(:            <h1>:)
(:                {$doc//tei:title/text()}:)
(:            </h1>:)
(:            <hr/>:)
(:        </div>:)
(:        <!-- metadata -->:)
(:        <div class="row">:)
(:        <!-- Identification aspects -->:)
(:        <div class="col-sm-4">:)
(:            <p>:)
(:                <span class="show-metadata">TM: </span>:)
(:                {$tm}:)
(:            </p>:)
(:            <p>:)
(:                <span class="show-metadata">Type: </span>:)
(:                {$type}:)
(:            </p>:)
(:            <p>:)
(:                <span class="show-metadata">Subtype: </span>:)
(:                {$subtype}:)
(:            </p>:)
(:            <p>:)
(:                <span class="show-metadata">Origin: </span>:)
(:                {show:place($doc)}:)
(:            </p>:)
(:            <p>:)
(:                <span class="show-metadata">Date: </span>:)
(:                {show:date($doc)}:)
(:            </p>:)
(:        </div>:)
(:        <!-- Physical aspects -->:)
(:        <div class="col-sm-4">:)
(:            <p>:)
(:                <span class="show-metadata">Width: </span>:)
(:                {$w} cm:)
(:            </p>:)
(:            <p>:)
(:                <span class="show-metadata">Height: </span>:)
(:                {$h} cm:)
(:            </p>:)
(:            <p>:)
(:                <span class="show-metadata">Shape: </span>:)
(:                {$shape, ' '} :)
(:                {if ($shape eq 'Vertical') then :)
(:                    <i class="far fa-file"/>:)
(:                else if ($shape eq 'Horizontal') then:)
(:                    <i class="far fa-file icon-rotate"/>:)
(:                else:)
(:                    <i class="far fa-square"/>:)
(:                }:)
(:            </p>:)
(:            <p>:)
(:                <span class="show-metadata">Fibres: </span>:)
(:                {if ($fibres eq 'fv') then :)
(:                    <span>Vertical <i class="fas fa-arrow-down"/></span>:)
(:                else:)
(:                    <span>Horizontal <i class="fas fa-arrow-right"/></span>:)
(:                }:)
(:            </p>:)
(:            <p>:)
(:                <span class="show-metadata">Seal: </span>:)
(:                {$doc//tei:seal/tei:p/text()}:)
(:            </p>:)
(:        </div>:)
(:        <!-- external links -->:)
(:        <div class="col-sm-4">:)
(:            <div class="btn-toolbar">:)
(:                <a id="ddb-hybrid" class="btn btn-info mx-1" role="button" target="_blank" href="{concat($URI:papinfo, $doc//tei:idno[@type='ddb-hybrid'])}">:)
(:                    papyri.info:)
(:                </a>:)
(:                <!-- display only one link to HGV, but there could be two, e.g. TM 13. -->:)
(:                <a class="btn btn-info mx-1" role="button" target="_blank" href="{concat($URI:hgv, $doc//tei:idno[@type='hgv'])}">:)
(:                    HGV:)
(:                </a>:)
(:                <!-- display button only if the link exists -->:)
(:                {if (not($img_link eq '')) then:)
(:                    <a class="img-btn" role="button" target="_blank" href="{$img_link}">:)
(:                        <i class="fas fa-camera fa-2x"/>:)
(:                    </a>:)
(:                else():)
(:                }:)
(:            </div>:)
(:        </div>:)
(:        </div>:)
(:        <hr/>:)
(:        <!-- papyrus schema -->:)
(:        <div class="row">:)
(:            <div class="col">:)
(:                <ul class="legend">:)
(:                <!-- list of text sections -->:)
(:                {:)
(:                    let $auth := doc(concat($config:data-root, '/authority-lists.xml'))//tei:list[@type eq 'layout']:)
(:                    for $section in $sections/tei:locus:)
(:                    let $id := functx:substring-after-last($section/@ana/string(), '#'):)
(:                    let $name := $auth/tei:item[@xml:id eq $id]/tei:label/string():)
(:                    return:)
(:                        <li><span class="{$id}"></span> {$name}</li>:)
(:                }:)
(:                </ul>:)
(:            </div>:)
(:        </div>:)
(:        <div class="row">:)
(:        <div class="col">:)
(:            {local:svg($w, $h, $text_sections/div)}:)
(:        </div>:)
(:        </div>:)
(:    </div>:)



(: PAST TESTS :)
(:  
declare function show:text-with-sections($text as node(), $sections as node()){
    let $newtext := xs:string($text)
    for $locus in $sections//tei:locus
        for $line in $locus/@from to $locus/@to
           let $section_name := functx:substring-after-last($locus/@ana/string(), '#')
           let $newline := xs:string(functx:add-or-update-attributes($text//*[@id eq concat('al', $line)], xs:QName('class'), $section_name))
           let $pattern := concat('<.*? id="al', $line, '".*?>')
           
    return
        replace($newtext, $pattern, $newline)
};
declare function show:text-with-sections($text as node(), $sections as node(), $lines as xs:integer){
    for $i in 1 to $lines
        for $line in $text//*[@id eq concat('al', $i)]
        
        for $locus in $sections/tei:locus
        where $i in $locus/@from to $locus/@to
    return
        $line
        (functx:add-or-update-attributes($text//*[@id eq concat('al', $line)], xs:QName('class'), $section_name), 
            $text//*[@id eq concat('al', $line)]/following-sibling::text()[1]))
};

(: testing text sections. let $text_sections := show:text-with-sections($text_html, $sections):)
let $doc := doc('/db/apps/grammateus/data/papyri/9098.xml')
let $text_xml := gramm:doc-expand-text($doc)
let $text_html := show:text($text_xml)
let $sections := $doc//tei:ab[@type eq 'section']



return
    $sections:)

(: testing search in origPlace. Arsinoites should return 112 docs :)

(:  
let $query := 'bakchias'

for $doc in collection($URI:papyri)//tei:origPlace[ft:query(.,$query)]

return
    <item>{$doc//tei:title}</item>:)


(:             let $links := concat($doc//tei:div[@type eq "figure"]//tei:figure/tei:graphic/@url/string(), 'and')
            for $figure in $doc//tei:div[@type eq "figure"]/tei:p/tei:figure
            return 
                $figure/tei:graphic/@url/string() :)
